# -*- coding: hebrew -*-
from typing import Any

from .utils import *

# Config
TEST_DATE = datetime.date(2024, 11, 1)


class Article:
    def __init__(self, name: str):
        """
        :param name: Case-sensitive
        """
        self.details: dict[str, Any] = get_article_data(name)

        # Map details to class
        self.name: str = self.details["title"]
        self.key: str = self.details["key"]
        self.id: str = self.details["id"]
        self.content_model: str = self.details["content_model"]
        self.license: dict = self.details["license"]
        self.latest: dict = self.details["latest"]
        self.html_url: str = self.details["html_url"]

    def __repr__(self):
        return f"Article({self.name}, {self.id})"

    def __eq__(self, other):
        return self.id == other.revision_id and self.key == other.key

    @overload
    def get_page(self, date: datetime.date, lang: str = "en"): ...
    @overload
    def get_page(self, lang: str = "en"): ...

    def get_page(self, *args, lang: str = "en"):
        if len(args) == 0:
            return Page(self.latest["id"], lang=lang)

        elif len(args) == 1 and type(args[0]) == datetime.date:
            return Page(self.key, args[0], lang=lang)

        else:
            raise AttributeError("Unexpected arguments")


class Page:
    """
    The difference between a wikipls.Page and a wikipls.Article:
    Article - Collection of all versions of all languages of all dates for a single article. A 'collection' of WikiPages
    Page - One specific version of an article, in a specific date and a specific language
    """

    # todo Make this accept also date: str

    memory: dict = {}
    @overload
    def __init__(self, name: str, date: datetime.date, lang="en"): ...
    @overload
    def __init__(self, page_id: int): ...

    def __init__(self, *args, lang=LANG):

        # Validate input
        if len(args) == 0:
            raise AttributeError("No arguments were provided")
        elif (len(args) > 2
              or len(args) == 1 and type(args[0]) != int
              or len(args) == 2 and (type(args[0]) != str or type(args[1]) != datetime.date)):
            raise AttributeError(f"Unexpected arguments. Args: {args}")

        using: str = "details" if len(args) == 2 else "id"
        identifier = args[0]

        # Get details
        if using == "details":
            date = args[1]
            self.article_details: dict[str, Any] = get_article_data(identifier, lang=lang)
            self.page_details: dict[str, Any] = get_page_data(identifier, date, lang=lang)
        else:  # using ID
            self.article_details: dict[str, Any] = get_article_data(identifier, lang=lang)
            self.page_details: dict[str, Any] = get_page_data(identifier, lang=lang)

        # self.title: Article = self.details["title"] # todo If got from Article object

        # Map details to class
        self.name: str = self.article_details["title"]
        self.key: str = self.article_details["key"]
        self.article_id: int = self.article_details["id"]
        self.lang: str = self.article_details["html_url"].removeprefix("https://")[:2]
        self.content_model: str = self.article_details["content_model"]
        self.license: dict = self.article_details["license"]

        self.revision_id: int = self.page_details["id"]
        self.date: datetime.date = from_timestamp(self.page_details["timestamp"])

    def __repr__(self):
        return f"Page({self.name}, {self.date}, {self.article_id})"

    def __eq__(self, other):
        return self.article_id == other.revision_id and self.key == other.key

    @property
    def views(self) -> int:
        if "views" not in self.memory:
            self.memory["views"]: int = get_views(self.key, self.date, self.lang)
        return self.memory["views"]

    @property
    def html(self) -> str:
        if "html" not in self.memory:
            self.memory["html"]: str = get_html(self.key)
        return self.memory["html"]

    @property
    def summary(self) -> str:
        if "summary" not in self.memory:
            self.memory["summary"]: str = get_summary(self.key)
        return self.memory["summary"]

    @property
    def media(self) -> tuple[dict, ...]:
        if "media" not in self.memory:
            self.memory["media"]: tuple[dict, ...] = get_media_details(self.key)
        return self.memory["media"]

    @property
    def as_pdf(self) -> bytes:
        if "pdf_code" not in self.memory:
            self.memory["pdf_code"]: bytes = get_pdf(self.key)
        return self.memory["pdf_code"]

    @property
    def data(self) -> dict[str, Any]:
        if "data" not in self.memory:
            self.memory["data"]: dict = get_page_data(self.key)
        return self.memory["data"]



from typing import overload

import requests
import json
import urllib.parse

from datetime import date

LANG = "en"
HEADERS = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64"}  # todo Check wiki's docs and change headers


def to_timestamp(date_: date) -> str:
    return date_.strftime("%Y%m%d")


@overload
def get_views(name: str, date_: date, lang: str = LANG) -> int: ...
@overload
def get_views(name: str, date_: str, lang: str = LANG) -> int: ...


def get_views(name: str, date_: str | date, lang: str = LANG) -> int:
    if isinstance(date_, date):
        date_ = to_timestamp(date_)
    elif not isinstance(date_, str):
        raise AttributeError("date_ must be a string or a datetime.date object")

    url = u"https://wikimedia.org/api/rest_v1/metrics/pageviews/per-article/" \
          u"{}.wikipedia.org/all-access/all-agents/{}/daily/{}/{}" \
        .format(lang.lower(), urllib.parse.quote(name), date_, date_)

    response = response_for(url)

    return response["items"][0]["views"]


def get_html(name: str) -> str:
    response = requests.get(f"https://en.wikipedia.org/api/rest_v1/page/html/{name}")

    if response.status_code == 200:
        return response.content.decode("utf-8")


def get_summary(name: str) -> str:
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/summary/{name}")

    if response:
        return response["extract"]


def get_media(name: str) -> tuple[dict, ...]:
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/media-list/{name}")

    if response:
        return tuple(response["items"])


def get_segments(name: str) -> str:
    # todo Add strict=False option that'll raise an error if response is None
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/segments/{name}")

    if response:
        return response["segmentedContent"]


def get_pdf(name: str) -> bytes:
    response = requests.get(f"https://en.wikipedia.org/api/rest_v1/page/pdf/{name}")

    if response.status_code == 200:
        return response.content


def get_page_data(name: str) -> dict:
    response = response_for(f"https://api.wikimedia.org/core/v1/wikipedia/en/page/{name}/bare")
    return response


def response_for(url: str) -> dict | None:
    response = requests.get(url, headers=HEADERS)
    result = json.loads(response.text)

    if response.status_code == 200:
        return result
    elif response.status_code == 400:
        raise AttributeError(f"One or more of the arguments given is invalid. "
                             f"\n{result['title']}: {result['detail']}")
    elif response.status_code == 404:
        if 'title' in result and 'detail' in result:
            raise Exception(f"No page was found. \n{result['title']}: {result['detail']}")
        elif 'messageTranslations' in result and 'en' in result['messageTranslations']:
            raise Exception(result["messageTranslations"]["en"])
    else:
        result = json.loads(response.text)
        print(f"New error: {response.status_code}, {result['title']}: {result['detail']}")



import requests
import json
import urllib.parse
import datetime

from typing import overload, Iterable


LANG = "en"
# HEADERS = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64"}  # todo Check wiki's docs and change headers
HEADERS = {
    'User-Agent': 'MediaWiki REST API docs examples/0.1 (https://www.mediawiki.org/wiki/API_talk:REST_API)'
}


def to_timestamp(date: datetime.date | str) -> str:
    if type(date) == datetime.date:
        return date.strftime("%Y%m%d")
    else:  # Convert from format yyyy-mm-ddThh:mm:ssZ
        return date.split('T')[0].replace('-', '')


def from_timestamp(timestamp: str) -> datetime.date:
    if "T" in timestamp:
        date_only: str = timestamp.split('T')[0]
        date_info: tuple[int] = tuple(int(info) for info in date_only.split('-'))
        return datetime.date(date_info[0], date_info[1], date_info[2])
    else:
        return datetime.date(int(timestamp[:5]), int(timestamp[5:7]), int(timestamp[7:9]))


@overload
def id_of_page(name: str, lang: str = LANG) -> int: ...
@overload
def id_of_page(name: str, date: str | datetime.date, lang: str = LANG) -> int: ...


def id_of_page(*args, lang: str = LANG):
    # Validate input
    if len(args) != 1 and len(args) != 2:
        raise AttributeError("Expected 1 or 2 arguments")
    elif type(args[0]) != str:
        raise AttributeError("name argument must be a string")
    elif len(args) == 2 and (type(args[1]) != str and type(args[1]) != datetime.date):
        raise AttributeError("date argument must be a string or a datetime.date object")
    elif type(lang) != str:
        raise AttributeError("lang key-argument must be a string")

    # Set-up arguments
    name = args[0]
    is_date: bool = len(args) == 2

    # Get ID from args
    if is_date:
        date = args[1]

        if type(date) == datetime.date:
            date = to_timestamp(date)

        url = f"https://{lang}.wikipedia.org/w/rest.php/v1/page/{name}/history"

        response = response_for(url)["revisions"]

        # Check timestamps
        for revision in response:
            formatted_timestamp = revision["timestamp"].split('T')[0].replace('-', '')

            if formatted_timestamp <= date:
                return revision["id"]

    else:
        response = response_for(f"https://api.wikimedia.org/core/v1/wikipedia/{lang}/page/{name}/bare")

        return response["id"]


def name_of_page(id: int, lang=LANG) -> str:
    id_details = response_for(f"http://{lang}.wikipedia.org/w/api.php",
                              params={"action": "query", "pageids": id, "format": "json"})

    if "title" in id_details["query"]["pages"][str(id)]:
        return id_details["query"]["pages"][str(id)]["title"]
    else:
        revision_details = response_for(f"https://{lang}.wikipedia.org/w/rest.php/v1/revision/{id}/bare")
        return revision_details["page"]["key"]


@overload
def get_views(name: str, date: datetime.date, lang: str = LANG) -> int: ...
@overload
def get_views(name: str, date: str, lang: str = LANG) -> int: ...


def get_views(name: str, date: str | datetime.date, lang: str = LANG) -> int:
    if isinstance(date, datetime.date):
        date = to_timestamp(date)
    elif not isinstance(date, str):
        raise AttributeError("date_ must be a string or a datetime.date object")

    url = u"https://wikimedia.org/api/rest_v1/metrics/pageviews/per-article/" \
          u"{}.wikipedia.org/all-access/all-agents/{}/daily/{}/{}" \
        .format(lang.lower(), urllib.parse.quote(name), date, date)

    response = response_for(url)

    return response["items"][0]["views"]


def get_html(name: str) -> str:
    response = requests.get(f"https://en.wikipedia.org/api/rest_v1/page/html/{name}")

    if response.status_code == 200:
        return response.content.decode("utf-8")


def get_summary(name: str) -> str:
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/summary/{name}")

    if response:
        return response["extract"]


def get_media_details(name: str) -> tuple[dict, ...]:
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/media-list/{name}")

    if response:
        return tuple(response["items"])


def get_image(details: dict[str, ...]) -> bytes:
    src_url = details["srcset"][-1]["src"]
    response = requests.get(f"https:{src_url}", headers=HEADERS)
    return response.content


@overload
def get_all_images(name: str, strict: bool = False) -> tuple[bytes]: ...
@overload
def get_all_images(details: Iterable[dict[str, ...]], strict: bool = False) -> tuple[bytes]: ...


def get_all_images(input: str | Iterable[dict[str, ...]], strict: bool = True) -> tuple[bytes]:
    if type(input) == "str":
        details: Iterable[dict[str, ...]] = get_media_details(input)
    else:
        details = input

    # Check for non-image media
    if strict:
        for media in details:
            if media["type"] != "image":
                raise AttributeError("Media list cannot contain media objects that are not images.")
    else:
        details = tuple(media for media in details if media["type"] == "image")

    all_images = tuple(get_image(image) for image in details)
    return all_images


def get_segments(name: str) -> str:
    # todo Add strict=False option that'll raise an error if response is None
    response = response_for(f"https://en.wikipedia.org/api/rest_v1/page/segments/{name}")

    if response:
        return response["segmentedContent"]


def get_pdf(name: str) -> bytes:
    response = requests.get(f"https://en.wikipedia.org/api/rest_v1/page/pdf/{name}")

    if response.status_code == 200:
        return response.content


@overload
def get_page_data(name: str, lang: str = LANG) -> dict[str, ...]: ...
@overload
def get_page_data(name: str, date: str | datetime.date, lang: str = LANG) -> dict[str, ...]: ...
@overload
def get_page_data(id: int, lang: str = LANG) -> dict[str, ...]: ...


def get_page_data(*args, lang: str = LANG) -> dict[str, ...]:
    # Validate arguments
    # You should read it as the rules for valid input (and avoid the "not"s in the beginning)
    if not (len(args) == 1 or len(args) == 2):
        raise AttributeError(f"Expected 1 or 2 arguments, got {len(args)}")
    elif not (type(args[0]) == str or type(args[0]) == int):
        raise AttributeError(f"name argument must be string or int. Got type {type(args[0])} instead")
    elif len(args) == 2 and not (type(args[1]) == datetime.date or type(args[1]) == str):
        raise AttributeError(f"date argument must be either string or datetime.date")

    is_date: bool = len(args) == 2
    by: str = "name" if type(args[0]) == str else "id"

    if by == "id":
        id = args[0]

    else:  # By name
        name = args[0]

        if is_date:
            date = args[1]
            id = id_of_page(name, date)
        else:
            id = id_of_page(name)

    revision_res = response_for(f"https://{lang}.wikipedia.org/w/rest.php/v1/revision/{id}/bare")
    revision_res.pop("page")
    revision_res.pop("user")
    return revision_res


def get_article_data(identifier: str | int, lang: str = LANG) -> dict[str, ...]:
    if type(identifier) == str:
        by = "name"
    else:
        by = "id"

    if by == "id":
        # Get article name using ID
        id_details = response_for(f"http://en.wikipedia.org/w/api.php",
                                  params={"action": "query", "pageids": identifier, "format": "json"})

        if "title" in id_details["query"]["pages"][str(identifier)]:
            name = id_details["query"]["pages"][str(identifier)]["title"]
        else:
            name = name_of_page(identifier)

    else:
        name = identifier

    response = response_for(f"https://{lang}.wikipedia.org/w/rest.php/v1/page/{name}/bare")

    out_details: dict[str, ...] = {
        "title": response["title"],
        "key": response["key"],
        "id": response["id"],
        "latest": response["latest"],
        "content_model": response["content_model"],
        "license": response["license"],
        "html_url": response["html_url"]
    }

    return out_details


@overload
def get_revision_data(name: str) -> dict[str, ...]: ...
@overload
def get_revision_data(name: str, date: str | datetime.date) -> dict[str, ...]: ...
@overload
def get_revision_data(id: int) -> dict[str, ...]: ...


def get_revision_data(*args, lang: str = LANG) -> dict[str, ...]:
    # Validate arguments
    # You should read it as the rules for valid input (and avoid the "not"s in the beginning)
    if not (len(args) == 1 or len(args) == 2):
        raise AttributeError(f"Expected 1 or 2 arguments, got {len(args)}")
    elif not (type(args[0]) == str or type(args[0]) == int):
        raise AttributeError(f"name argument must be string or int. Got type {type(args[0])} instead")
    elif len(args) == 2 and not (type(args[1]) == datetime.date or type(args[1]) == str):
        raise AttributeError(f"date argument must be either string or datetime.date")

    if type(args[0]) == str:
        by = "name"
    else:
        by = "id"

    is_date: bool = len(args) == 2
    # if type(args[0] == str):
    #     name = args[0]
    # else:
    #     id = args[0]

    if by == "id":
        id = args[0]

    else:   # By name
        name = args[0]

        if is_date:
            date = args[1]
            id = id_of_page(name, date)

        else:
            id = id_of_page(name)

    response = response_for(f"https://{lang}.wikipedia.org/w/rest.php/v1/revision/{id}/bare")
    return response

    # if is_date:
    #     date = args[1]
    #     if type(date) == str:
    #
    #         pass
    #         # response = response_for()


def response_for(url: str, params: dict | None = None) -> dict | None:
    response = requests.get(url, headers=HEADERS, params=params)
    result = json.loads(response.text)

    # Handle response errors
    if response.status_code == 200:
        return result
    elif response.status_code == 400:
        raise AttributeError(f"One or more of the arguments given is invalid. "
                             f"\n{result['title']}: {result['detail']}")
    elif response.status_code == 404:
        if 'title' in result and 'detail' in result:
            raise Exception(f"No page was found for {url}. \n{result['title']}: {result['detail']}")
        elif 'messageTranslations' in result and 'en' in result['messageTranslations']:
            raise Exception(result["messageTranslations"]["en"])
    else:
        result = json.loads(response.text)
        print(f"New error: {response.status_code}, {result['title']}: {result['detail']}")



# -*- coding: hebrew -*-
from typing import Any

from .utils import *

# Config
TEST_DATE = date(2024, 11, 1)


class Article:
    def __init__(self, name: str):
        """
        :param name: Case-sensitive
        """
        self.details: dict[str, Any] = get_page_data(name)

        # Map details to class
        self.id: int = self.details["id"]
        self.title: str = self.details["title"]
        self.key: str = self.details["key"]
        self.content_model: str = self.details["content_model"]
        self.license: dict = self.details["license"]
        self.latest: dict = self.details["latest"]
        self.html_url: str = self.details["html_url"]

    def __repr__(self):
        return f"Article({self.title}, {self.id})"

    def __eq__(self, other):
        return self.id == other.id and self.key == other.key

    def get_page(self, date_: date, lang: str = "en"):
        return Page(self, date_, lang)

        # todo Revisions


class Page:
    """
    The difference between a wikipy.Page and a wikipy.Article:
    Article - Collection of all versions of all languages of all dates for a single article. A 'collection' of WikiPages
    Page - One specific version of an article, in a specific date and a specific language
    """

    memory: dict = {}

    def __init__(self, article: Article, date_: date, lang="en"):
        self.from_article: Article = article
        self.name = self.from_article.key
        self.date: date = date_
        self.lang: str = lang

        self.details: dict[str, Any] = get_page_data(article.key)

        # Map details to class
        self.id: int = self.details["id"]
        self.title: str = self.details["title"]
        self.key: str = self.details["key"]
        self.content_model: str = self.details["content_model"]
        self.license: dict = self.details["license"]
        self.latest: dict = self.details["latest"]
        self.html_url: str = self.details["html_url"]

    def __repr__(self):
        return f"Page({self.title}, {self.date}, {self.id})"

    def __eq__(self, other):
        return self.id == other.id and self.key == other.key

    @property
    def views(self) -> int:
        if "views" not in self.memory:
            self.memory["views"]: int = get_views(self.name, self.date, self.lang)
        return self.memory["views"]

    @property
    def html(self) -> str:
        if "html" not in self.memory:
            self.memory["html"]: str = get_html(self.name)
        return self.memory["html"]

    @property
    def summery(self) -> str:
        if "summery" not in self.memory:
            self.memory["summery"]: str = get_summary(self.name)
        return self.memory["summery"]

    @property
    def media(self) -> tuple[dict, ...]:
        if "media" not in self.memory:
            self.memory["media"]: tuple[dict, ...] = get_media(self.name)
        return self.memory["media"]

    @property
    def as_pdf(self) -> bytes:
        if "pdf_code" not in self.memory:
            self.memory["pdf_code"]: bytes = get_pdf(self.name)
        return self.memory["pdf_code"]

    @property
    def data(self) -> dict[str, Any]:
        if "data" not in self.memory:
            self.memory["data"]: dict = get_page_data(self.name)
        return self.memory["data"]
